/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg;

import io.restassured.RestAssured;
import io.restassured.http.ContentType;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import static io.restassured.RestAssured.given;
import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.hasSize;
import org.skyscreamer.jsonassert.JSONAssert;
import org.skyscreamer.jsonassert.JSONCompareMode;
import java.io.File;
import java.nio.charset.StandardCharsets;
import static net.javacrumbs.jsonunit.JsonMatchers.jsonEquals;
import net.javacrumbs.jsonunit.core.Option;
import org.assertj.core.util.Files;
import static uk.co.datumedge.hamcrest.json.SameJSONAs.sameJSONAs;

/**
 *
 * @author omozegieaziegbe
 */
public class SessionInfoTest {

    @BeforeAll
    public static void setup() {
        RestAssured.baseURI = "http://localhost:8080/api";
    }

    @Test
    public void testSessionInfoResponse() {
        given()
                .contentType(ContentType.JSON)
                .when()
                .get("/session-info")
                .then()
                .statusCode(200)
                .body("activeSession", equalTo(true))
                .body("token", equalTo("abc123"))
                .body("details", hasSize(1))
                .body("details[0].accountType", equalTo(2))
                .body("details[0].accountDescription", equalTo("Premium"))
                .body("details[0].username", equalTo("mr_fish"))
                .body("details[0].clientId", equalTo(42));
    }

    @Test
    public void testWholeJsonBody() {
        // Extract the JSON response body as a string
        String body = given()
                .get("/session-info")
                .then()
                .extract()
                .body()
                .asString();

        // Assert the entire JSON structure as a single string
        assertThat(body)
                .isEqualTo("{\"details\":[{\"accountDescription\":\"Premium\",\"clientId\":42,\"username\":\"mr_fish\",\"accountType\":2}],\"activeSession\":true,\"token\":\"abc123\"}");
    }

    @Test
    void whenGetBody_thenCanCompareByWholeString() throws Exception {
        // Extract the JSON response body as a string
        String body = given()
                .get("/session-info")
                .then()
                .extract()
                .body()
                .asString();

        // Expected JSON string
        String expectedJson = "{\"activeSession\":true,\"token\":\"abc123\",\"details\":[{\"accountType\":2,\"accountDescription\":\"Premium\",\"username\":\"mr_fish\",\"clientId\":42}]}";

        // Assert JSON, ignoring field order
        JSONAssert.assertEquals(expectedJson, body, JSONCompareMode.LENIENT);
    }

    @Test
    void assertJsonWithExtraFields_JsonAssert() {
        given()
                .get("/session-info")
                .then()
                .body(sameJSONAs(Files.contentOf(new File("src/test/resources/expected.json"), StandardCharsets.UTF_8))
                        .allowingExtraUnexpectedFields());
    }
    
    @Test
    void compareFieldsWithJsonUnitIgnoringExtras() {
        given()
            .get("/session-info")
            .then()
            .body(jsonEquals(Files.contentOf(new File("src/test/resources/expected.json"), StandardCharsets.UTF_8))
            .when(Option.IGNORING_EXTRA_FIELDS));
    }

}
